using System;
using System.Diagnostics;
using System.IO;
using System.Windows.Forms;
using System.ComponentModel;
using System.Threading;
using System.Drawing;
using System.Collections.Generic;

namespace B4ALogViewer
{
    public partial class StartupForm : Form
    {
        string versionedtitle = "Basic4Android Log Viewer v1.4";
        // 1.4 if you do adb logcat -b main -v time some messages seem to be omitted
        //     if you do adb logcat  -v time it seems to work fine
        //     so btnStart_Click modified to omit -b parameter for the main buffer

        List<string> devices = new List<string>();
        StreamReader outputReader = null;
        StreamReader errorReader = null;
        Process process = new Process();
        bool processStarted;
        private Thread thread;
        private delegate void guiwriterdel(char c, string t, string s);
        private guiwriterdel guiwriter;


        [Flags]
        private enum MSGTYPE
        {
            NONE,
            VERBOSE = 1,
            DEBUG = 2,
            INFO = 4,
            WARNING = 8,
            ERROR = 16
        }
        private MSGTYPE msgtypes = (MSGTYPE) 0x1f;
        private string[] msgtags;
        private bool started;


        public StartupForm()
        {
            InitializeComponent();
            guiwriter = guiwritecode;
            this.Text = versionedtitle;
            msgtags = new string[0];
            cmboLog.SelectedIndex = 0;
        }


        private void btnStart_Click(object sender, EventArgs e)
        {
            if (started)
            {
                cmboLog.Enabled = true;
                btnStart.Text = "Start";
                started = false;
                close();
                return;
            }
            cmboLog.Enabled = false;
            btnStart.Text = "Stop";
            started = true;
            txtResult.Text = string.Empty;
            {
                outputReader = null;
                try
                {
                    devices.Clear();
                    ProcessStartInfo processStartInfo =
                            new ProcessStartInfo("adb", "devices");
                    processStartInfo.ErrorDialog = false;
                    processStartInfo.UseShellExecute = false;
                    processStartInfo.RedirectStandardInput = true;
                    processStartInfo.RedirectStandardOutput = true;
                    processStartInfo.RedirectStandardError = true;
                    processStartInfo.CreateNoWindow = true;
                    process = new Process();
                    process.StartInfo = processStartInfo;
                    processStarted = process.Start();
                    if (processStarted)
                    {
                        outputReader = process.StandardOutput;
                        string msg = outputReader.ReadToEnd();
                        string[] lines = msg.Split(new char[]{ '\n'});
                        for (int i = 1; i < lines.Length; i++)
                        {
                            int sp = lines[i].IndexOf('\t');
                            if (sp > 0)
                                devices.Add(lines[i].Substring(0, sp));
                        }
                    }
                    else
                        throw new Exception("Cannot start adb.exe!");
                    if (devices.Count == 0)
                        throw new Exception("No devices connected!");
                    string device = devices[0];
                    if (devices.Count > 1)
                    {
                        ChooseForm dlg = new ChooseForm();
                        for (int i = 0; i < devices.Count; i++)
                            dlg.adddevice(devices[i]);
                        dlg.ShowDialog();
                        device = dlg.selected;
                    }
                    processStartInfo.FileName = "adb";
                    //processStartInfo.Arguments = "-s " + device + " logcat -b " + ((string)cmboLog.SelectedItem).ToLower() + " -v time";
                    if (((string)cmboLog.SelectedItem).ToLower() == "main")
                        processStartInfo.Arguments = "-s " + device + " logcat -v time" ;
                    else
                        processStartInfo.Arguments = "-s " + device + " logcat -b " + ((string)cmboLog.SelectedItem).ToLower() + " -v time";

                    process = new Process();
                    process.StartInfo = processStartInfo;
                    processStarted = process.Start();
                    if (processStarted)
                    {
                        this.Text = versionedtitle + " - " + device;
                        thread = new Thread(threadcode);
                        thread.Start();
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }


        private void threadcode()
        {
            //Get the output stream
            outputReader = process.StandardOutput;
            errorReader = process.StandardError;
            while (true)
            {
                string tag = "";
                string msg = outputReader.ReadLine() + "\n";
                int start = msg.IndexOf('/') + 1; // tag start
                if (start > 0)
                {
                    int end = msg.IndexOf('(');
                    char typechar = msg[start-2]; // type of message
                    if (end > 0)
                        tag = msg.Substring(start, end - start).ToLower().Trim();
                    bool matched = !checkBoxF.Checked;
                    for (int i = 0; i < msgtags.Length; i++)
                    {
                        if (msgtags[i] == tag)
                        {
                            matched = true;
                            break;
                        }
                    }
                    if (matched)
                    {
                        if ((msgtypes & MSGTYPE.VERBOSE) != 0 && typechar == 'V'
                            || (msgtypes & MSGTYPE.DEBUG) != 0 && typechar == 'D'
                            || (msgtypes & MSGTYPE.INFO) != 0 && typechar == 'I'
                            || (msgtypes & MSGTYPE.WARNING) != 0 && typechar == 'W'
                            || (msgtypes & MSGTYPE.ERROR) != 0 && typechar == 'E'
                            || typechar == 'F')
                            this.Invoke(guiwriter, new object[] { typechar, tag, msg });
                    }
                }
            }
        }


        private void guiwritecode(char type, string tag, string text)
        {
            Font oldfont = txtResult.Font;
            Color col = Color.Black;
            switch (type)
            {
                case 'D': col = Color.Blue; break;
                case 'I': col = Color.Green; break;
                case 'W': col = Color.DarkOrange; break;
                case 'E': col = Color.Red; break;
                case 'F': col = Color.DarkRed; break;
            }
            if (tag == "b4a" && checkBoxH.Checked)
            {
                col = Color.Black;
                //txtResult.SelectionFont = new Font(oldfont, FontStyle.Bold);
            }
            if (!checkBoxT.Checked)
                text = text.Substring(19);
            else
                text = text.Substring(6);

            txtResult.SelectionStart = txtResult.Text.Length;
            txtResult.SelectionColor = col;
            txtResult.AppendText(text);
            txtResult.ScrollToCaret();
        }

        private void close()
        {
            if (thread != null)
            {
                thread.Abort();
                thread = null;
            }
            if (outputReader != null)
            {
                outputReader.Close();
                outputReader = null;
            }
            if (processStarted)
            {
                try
                {
                    process.Kill();
                    processStarted = false;
                }
                catch
                {
                }
            }
        }


        protected override void  OnClosing(CancelEventArgs e)
        {
            close();
            base.OnClosing(e);
        }

        private void checkBoxV_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxV.Checked)
                msgtypes |= MSGTYPE.VERBOSE;
            else
                msgtypes &= ~MSGTYPE.VERBOSE;
        }

       private void checkBoxD_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxD.Checked)
                msgtypes |= MSGTYPE.DEBUG;
            else
                msgtypes &= ~MSGTYPE.DEBUG;
        }

        private void checkBoxI_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxI.Checked)
                msgtypes |= MSGTYPE.INFO;
            else
                msgtypes &= ~MSGTYPE.INFO;
        }

        private void checkBoxW_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxW.Checked)
                msgtypes |= MSGTYPE.WARNING;
            else
                msgtypes &= ~MSGTYPE.WARNING;
        }

        private void checkBoxE_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxE.Checked)
                msgtypes |= MSGTYPE.ERROR;
            else
                msgtypes &= ~MSGTYPE.ERROR;
        }

        private void checkBoxF_CheckedChanged(object sender, EventArgs e)
        {
        }

        private void txtTag_TextChanged(object sender, EventArgs e)
        {
            msgtags = txtTag.Text.Split(',');
            for (int i = 0; i < msgtags.Length; i++)
                msgtags[i] = msgtags[i].ToLower().Trim();
        }

        private void txtResult_Enter(object sender, EventArgs e)
        {
            if (started)
                txtTag.Focus();
        }

        private void btnB4A_Click(object sender, EventArgs e)
        {
            txtTag.Text = "B4A";
            checkBoxV.Checked = false;
            checkBoxD.Checked = false;
            checkBoxI.Checked = true;
            checkBoxW.Checked = false;
            checkBoxE.Checked = false;
            checkBoxF.Checked = true;
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            txtResult.Text = "";

        }



    }
}